/*
 * DISTRHO Cardinal Plugin
 * Copyright (C) 2021-2023 Filipe Coelho <falktx@falktx.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 3 of
 * the License, or any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * For a full copy of the GNU General Public License see the LICENSE file.
 */

#include <cstdint>

static constexpr const unsigned long sCardinalX11Icon[] = {
  // 16x16
  16, 16,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x2dffffff, 0x91ffffff, 0xd5ffffff, 0xf4ffffff, 0xf4ffffff, 0xd5ffffff, 0x91ffffff, 0x2dffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x003e3e3e, 0x06ffffff, 0x96ffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x95ffffff, 0x06ffffff, 0x003e3e3e, 0x00ffffff,
  0x00ffffff, 0x06ffffff, 0xbdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfbffffff, 0xfbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbdffffff, 0x06ffffff, 0x00ffffff,
  0x00ffffff, 0x96ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa1ffffff, 0x32ffffff, 0x02ffffff, 0x02ffffff, 0x32ffffff, 0xa2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x93ffffff, 0x00ffffff,
  0x2dffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0x65ffffff, 0x003b3b3b, 0x00535353, 0x10ffffff, 0x11ffffff, 0x004f4f4f, 0x003e4d61, 0x67ffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x2bffffff,
  0x91ffffff, 0xffffffff, 0xffffffff, 0xa1ffffff, 0x00763d3d, 0x18ffffff, 0xb8ffffff, 0xffffffff, 0xffffffff, 0xb7ffffff, 0x17ffffff, 0x00394a62, 0xa3ffffff, 0xffffffff, 0xffffffff, 0x8fffffff,
  0xd5ffffff, 0xffffffff, 0xffffffff, 0x31ffffff, 0x001a1a1a, 0xb9ffffff, 0xeeffffff, 0x6fffffff, 0x70ffffff, 0xefffffff, 0xb7ffffff, 0x003d567a, 0x33ffffff, 0xffffffff, 0xffffffff, 0xd4ffffff,
  0xf3ffffff, 0xffffffff, 0xfbffffff, 0x02ffffff, 0x11ffffff, 0xfeffffff, 0x6fffffff, 0x00393535, 0x00393535, 0x70ffffff, 0xfeffffff, 0x10ffffff, 0x03ffffff, 0xfbffffff, 0xffffffff, 0xf3ffffff,
  0xf3ffffff, 0xffffffff, 0xfcffffff, 0x03ffffff, 0x11ffffff, 0xfeffffff, 0x70ffffff, 0x00363333, 0x00363333, 0x71ffffff, 0xfeffffff, 0x10ffffff, 0x03ffffff, 0xfcffffff, 0xffffffff, 0xf1ffffff,
  0xd6ffffff, 0xffffffff, 0xffffffff, 0x32ffffff, 0x001a1a1a, 0xb7ffffff, 0xefffffff, 0x71ffffff, 0x71ffffff, 0xefffffff, 0xb5ffffff, 0x003e5b83, 0x33ffffff, 0xffffffff, 0xffffffff, 0xd4ffffff,
  0x91ffffff, 0xffffffff, 0xffffffff, 0xa1ffffff, 0x00783f3f, 0x18ffffff, 0xb7ffffff, 0xfeffffff, 0xfeffffff, 0xb5ffffff, 0x17ffffff, 0x003c5477, 0xa4ffffff, 0xffffffff, 0xffffffff, 0x8effffff,
  0x2dffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0x67ffffff, 0x00363636, 0x004f4f4f, 0x10ffffff, 0x10ffffff, 0x00464646, 0x003d4c5f, 0x68ffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x2bffffff,
  0x00ffffff, 0x94ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa3ffffff, 0x32ffffff, 0x02ffffff, 0x03ffffff, 0x34ffffff, 0xa4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x92ffffff, 0x00ffffff,
  0x00ffffff, 0x06ffffff, 0xbdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfbffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbcffffff, 0x06ffffff, 0x00ffffff,
  0x00ffffff, 0x003e3e3e, 0x06ffffff, 0x93ffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x91ffffff, 0x06ffffff, 0x003e3e3e, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x2bffffff, 0x8effffff, 0xd3ffffff, 0xf3ffffff, 0xf3ffffff, 0xd3ffffff, 0x8effffff, 0x2bffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,

  // 48x48
  48, 48,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b,
  0x002b2b2b, 0x18ffffff, 0x58ffffff, 0x90ffffff, 0xb8ffffff, 0xd9ffffff, 0xeeffffff, 0xf9ffffff, 0xf9ffffff, 0xeeffffff, 0xd9ffffff, 0xb7ffffff, 0x8fffffff, 0x57ffffff, 0x17ffffff, 0x002b2b2b,
  0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x002b2b2b, 0x001a1a1a, 0x00222222, 0x001a1a1a, 0x00181818, 0x00212121, 0x00212121, 0x00212121, 0x00212121, 0x13ffffff, 0x6dffffff,
  0xcaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc9ffffff,
  0x6cffffff, 0x13ffffff, 0x00212121, 0x00212121, 0x00212121, 0x00212121, 0x00181818, 0x001a1a1a, 0x00222222, 0x001a1a1a, 0x002b2b2b, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x001a1a1a, 0x00181818, 0x001e1e1e, 0x001b1b1b, 0x001b1b1b, 0x001d1d1d, 0x001c1c1c, 0x001b1b1b, 0x1bffffff, 0x90ffffff, 0xf5ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xf4ffffff, 0x8effffff, 0x1affffff, 0x001b1b1b, 0x001c1c1c, 0x001d1d1d, 0x001b1b1b, 0x001b1b1b, 0x001e1e1e, 0x00181818, 0x001a1a1a, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x00151515, 0x001a1a1a, 0x001d1d1d, 0x003b3017, 0x00785a14, 0x00997715, 0x00966f12, 0x05ffffff, 0x6fffffff, 0xf3ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xf5ffffff, 0x80ffffff, 0x06ffffff, 0x00815206, 0x007c5106, 0x006b490b, 0x003a2c14, 0x001d1d1d, 0x001a1a1a, 0x00151515, 0x00000000, 0x00000000, 0x00ffffff,
  0x00000000, 0x00000000, 0x001a1a1a, 0x001a1a1a, 0x001d1d1d, 0x005f4110, 0x00a76f0d, 0x00bc7d0f, 0x00cc6d0e, 0x27ffffff, 0xd0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xceffffff, 0x25ffffff, 0x00984a00, 0x00a06100, 0x00a26601, 0x006e4c0f, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00181818, 0x001d1d1d, 0x0056360d, 0x00996008, 0x00c13a06, 0x00d10200, 0x52ffffff, 0xf3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf2ffffff, 0x50ffffff, 0x00820200, 0x00933600, 0x00ac7408, 0x00775f20, 0x001d1d1d, 0x00181818, 0x00000000, 0x00000000,
  0x00000000, 0x001a1a1a, 0x001e1e1e, 0x002e2015, 0x00854a05, 0x00b93104, 0x00d00000, 0x60ffffff, 0xfbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x68ffffff, 0x007e0000, 0x00953b05, 0x00bb9025, 0x00443b22, 0x001e1e1e, 0x001a1a1a, 0x00000000,
  0x00000000, 0x00222222, 0x001b1b1b, 0x004e2d0b, 0x008c4a05, 0x00cf0200, 0x67ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x66ffffff, 0x007b0200, 0x00bd8f26, 0x00897732, 0x001b1b1b, 0x00222222, 0x00000000,
  0x00333333, 0x001b1b1b, 0x001b1b1b, 0x005e3308, 0x009a3b05, 0x53ffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe9ffffff, 0xddffffff, 0xddffffff, 0xe9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x50ffffff, 0x00ae711f, 0x00a38e39, 0x001b1b1b, 0x001a1a1a, 0x00333333,
  0x00333333, 0x00242424, 0x001d1d1d, 0x00643507, 0x27ffffff, 0xf3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xddffffff, 0x85ffffff, 0x3fffffff, 0x0effffff, 0x00262626, 0x00262626, 0x00262626, 0x00262626, 0x0effffff, 0x40ffffff, 0x86ffffff, 0xdfffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf2ffffff, 0x25ffffff, 0x00ad973c, 0x001d1d1d, 0x00242424, 0x00333333,
  0x002b2b2b, 0x00232323, 0x001c1c1c, 0x06ffffff, 0xd0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xceffffff, 0x4effffff, 0x01ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x00303030, 0x00303f30, 0x01ffffff, 0x4fffffff, 0xd0ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcfffffff, 0x05ffffff, 0x001c1c1c, 0x00212121, 0x002b2b2b,
  0x002b2b2b, 0x00212121, 0x001b1b1b, 0x78ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff, 0x72ffffff,
  0x03ffffff, 0x002c2c2c, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x00303030, 0x00294929, 0x0000ff00, 0x0000ff00, 0x04ffffff,
  0x75ffffff, 0xf9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x75ffffff, 0x001b1b1b, 0x00212121, 0x002b2b2b,
  0x002b2b2b, 0x00212121, 0x1bffffff, 0xf4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe9ffffff, 0x36ffffff, 0x001a1a1a,
  0x001a1a1a, 0x002c2c2c, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x00303030, 0x002a462a, 0x0000e000, 0x0000e000, 0x0000e000,
  0x0000e000, 0x39ffffff, 0xebffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf3ffffff, 0x19ffffff, 0x00212121, 0x002b2b2b,
  0x002b2b2b, 0x00212121, 0x90ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeaffffff, 0x28ffffff, 0x00895151, 0x00433333,
  0x001a1a1a, 0x002c2c2c, 0x00424242, 0x00797979, 0x00575757, 0x005f5f5f, 0x003f3f3f, 0x005a5a5a, 0x006b6b6b, 0x005d5d5d, 0x003f3f3f, 0x00303030, 0x002a452a, 0x0000d800, 0x0000d800, 0x0000d800,
  0x0000d800, 0x0000d100, 0x2affffff, 0xebffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8dffffff, 0x00212121, 0x002b2b2b,
  0x002b2b2b, 0x14ffffff, 0xf5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff, 0x36ffffff, 0x009f9f9f, 0x00b7b1b1, 0x007b3e3e,
  0x00272525, 0x002c2c2c, 0x00616161, 0x00505050, 0x003c3c3c, 0x1dffffff, 0x56ffffff, 0x6fffffff, 0x6fffffff, 0x56ffffff, 0x1cffffff, 0x00303030, 0x00294929, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000f800, 0x00000000, 0x38ffffff, 0xf9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf4ffffff, 0x13ffffff, 0x002b2b2b,
  0x002b2b2b, 0x6effffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x72ffffff, 0x00767676, 0x00727272, 0x00a2a2a2, 0x00936262,
  0x00463939, 0x002c2c2c, 0x003c3c3c, 0x4cffffff, 0xc9ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0xc8ffffff, 0x4cffffff, 0x00108810, 0x00108810, 0x00108810,
  0x00108810, 0x00118511, 0x001f1f1f, 0x001f1f1f, 0x74ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x6bffffff, 0x002b2b2b,
  0x002b2b2b, 0xcbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xceffffff, 0x03ffffff, 0x007d7d7d, 0x00787878, 0x00a5a5a5, 0x00864e4e,
  0x00413737, 0x0dffffff, 0xb0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xaeffffff, 0x0cffffff, 0x0002816b,
  0x00030303, 0x00030303, 0x00030303, 0x00030303, 0x04ffffff, 0xd1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc8ffffff, 0x002b2b2b,
  0x16ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4dffffff, 0x00ab9292, 0x00adadad, 0x00aaaaaa, 0x00907e7e, 0x006d3939,
  0x0dffffff, 0xcfffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xceffffff, 0x0dffffff,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00161616, 0x51ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x17ffffff,
  0x58ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdeffffff, 0x01ffffff, 0x00653b3b, 0x00864f4f, 0x00804c4c, 0x006b3d3d, 0x00372e2e,
  0xb0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfaffffff, 0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xaeffffff,
  0x00005042, 0x00005042, 0x00005042, 0x0000483b, 0x00161616, 0x02ffffff, 0xe0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x56ffffff,
  0x90ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x85ffffff, 0x001a1a1a, 0x001b1b1b, 0x002b2929, 0x002f2c2c, 0x001c1c1c, 0x4dffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x9effffff, 0x30ffffff, 0x003b3b3b, 0x01ffffff, 0x30ffffff, 0xa0ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0x4affffff, 0x0000ffd2, 0x0000ffd2, 0x0000e8bf, 0x00161616, 0x00262626, 0x88ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8effffff,
  0xb7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x40ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xc8ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x61ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x64ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xc7ffffff, 0x0000ffd2, 0x0000ffd2, 0x0000e8bf, 0x00161616, 0x00262626, 0x42ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb6ffffff,
  0xdbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x0effffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x1dffffff, 0xfeffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0x9effffff, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x002a2a2a, 0xa0ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xfeffffff, 0x1cffffff, 0x00333333, 0x00333333, 0x00333333, 0x00232323, 0x10ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdaffffff,
  0xecffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeaffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x55ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0x2fffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x31ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x53ffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x00533a3a, 0xecffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff,
  0xf9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdbffffff, 0x00494141, 0x004a4141, 0x004a4141, 0x004a4141, 0x004a4141, 0x6fffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xfaffffff, 0x02ffffff, 0x004a4141, 0x004a4141, 0x004a4141, 0x004a4141, 0x004a4141, 0x004a4141, 0x004a4141, 0x004a4141, 0x02ffffff, 0xfaffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x6effffff, 0x004a4141, 0x004a4141, 0x00494141, 0x00494141, 0x00504343, 0xdcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff,
  0xf9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdbffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x6fffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xfaffffff, 0x02ffffff, 0x00232323, 0x00232323, 0x00232323, 0x00232323, 0x00232323, 0x00232323, 0x00232323, 0x001e1e1e, 0x02ffffff, 0xfaffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x6effffff, 0x001f1f1f, 0x001f1f1f, 0x001f1f1f, 0x001b1b1b, 0x00523939, 0xdcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff,
  0xecffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeaffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x55ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0x30ffffff, 0x003c3c3c, 0x003c3c3c, 0x003c3c3c, 0x003c3c3c, 0x003c3c3c, 0x003c3c3c, 0x003c3c3c, 0x00313131, 0x32ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x53ffffff, 0x00232323, 0x00232323, 0x002d2d2d, 0x00262626, 0x00523939, 0xecffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff,
  0xdbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x0effffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x1dffffff, 0xfeffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0x9fffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x00303030, 0xa2ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xfeffffff, 0x1bffffff, 0x00000000, 0x00000000, 0x00161616, 0x00262626, 0x10ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd9ffffff,
  0xb7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x41ffffff, 0x001a1a1a, 0x00212020, 0x003d3030, 0x00403232, 0x00262424, 0xc8ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x64ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x66ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xc6ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00161616, 0x00262626, 0x43ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb5ffffff,
  0x8fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x86ffffff, 0x00342929, 0x00835050, 0x00c49696, 0x00bd9090, 0x00844e4e, 0x4bffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0xa0ffffff, 0x31ffffff, 0x01ffffff, 0x01ffffff, 0x32ffffff, 0xa2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0x4affffff, 0x0000c900, 0x00000000, 0x00000000, 0x00161616, 0x00262626, 0x89ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8effffff,
  0x57ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdeffffff, 0x01ffffff, 0x00d2c4c4, 0x00a3a3a3, 0x00a0a0a0, 0x00b7b0b0, 0x00793c3c,
  0xaeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfaffffff, 0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xadffffff,
  0x0000ff00, 0x0000f800, 0x00000000, 0x00000000, 0x00161616, 0x02ffffff, 0xe0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x55ffffff,
  0x16ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x4fffffff, 0x00adadad, 0x00777777, 0x00737373, 0x00a2a2a2, 0x00946262,
  0x0dffffff, 0xceffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcdffffff, 0x0cffffff,
  0x000e8e0e, 0x000f8b0f, 0x001d1d1d, 0x001d1d1d, 0x00282828, 0x53ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x15ffffff,
  0x002b2b2b, 0xc9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd0ffffff, 0x04ffffff, 0x007c7c7c, 0x00777777, 0x00a5a5a5, 0x00874f4f,
  0x00423838, 0x0dffffff, 0xaeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xacffffff, 0x0cffffff, 0x00037e68,
  0x00050505, 0x00050505, 0x00050505, 0x00050505, 0x04ffffff, 0xd3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc7ffffff, 0x002b2b2b,
  0x002b2b2b, 0x6cffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x75ffffff, 0x00acacac, 0x00a9a9a9, 0x00928282, 0x006e3939,
  0x00222222, 0x002c2c2c, 0x00696969, 0x4affffff, 0xc7ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0xc6ffffff, 0x49ffffff, 0x0000ffd2, 0x0000ffd2, 0x00008870,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x77ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x69ffffff, 0x002b2b2b,
  0x002b2b2b, 0x13ffffff, 0xf4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf9ffffff, 0x38ffffff, 0x00835050, 0x006b3b3b, 0x00392f2f,
  0x001a1a1a, 0x002c2c2c, 0x00838383, 0x008e8e8e, 0x00545454, 0x1bffffff, 0x53ffffff, 0x6effffff, 0x6effffff, 0x53ffffff, 0x1bffffff, 0x002f2f2f, 0x002b423e, 0x0000977d, 0x0000977d, 0x00007661,
  0x00005042, 0x00005042, 0x00005042, 0x3affffff, 0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf3ffffff, 0x12ffffff, 0x002b2b2b,
  0x002b2b2b, 0x00212121, 0x8effffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff, 0x2affffff, 0x001d1d1d, 0x001a1a1a,
  0x001a1a1a, 0x002c2c2c, 0x00474747, 0x004e4e4e, 0x003b3b3b, 0x00454545, 0x004f4f4f, 0x003a3a3a, 0x00424242, 0x00515151, 0x003b3b3b, 0x00303030, 0x00294943, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x2affffff, 0xebffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8bffffff, 0x00212121, 0x002b2b2b,
  0x002b2b2b, 0x00212121, 0x1affffff, 0xf3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff, 0x39ffffff, 0x001a1a1a,
  0x001a1a1a, 0x002c2c2c, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x00303030, 0x00294943, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x3affffff, 0xecffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf2ffffff, 0x18ffffff, 0x00212121, 0x002b2b2b,
  0x002b2b2b, 0x00212121, 0x001b1b1b, 0x75ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf9ffffff, 0x74ffffff,
  0x04ffffff, 0x00282828, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x002a2a2a, 0x00303131, 0x00313938, 0x00313938, 0x04ffffff,
  0x77ffffff, 0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x73ffffff, 0x001b1b1b, 0x00212121, 0x002b2b2b,
  0x002b2b2b, 0x00232323, 0x001c1c1c, 0x05ffffff, 0xceffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xd1ffffff, 0x50ffffff, 0x01ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x02ffffff, 0x52ffffff, 0xd3ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcdffffff, 0x05ffffff, 0x001c1c1c, 0x00212121, 0x002b2b2b,
  0x00333333, 0x00242424, 0x001d1d1d, 0x0081580d, 0x25ffffff, 0xf1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xdfffffff, 0x87ffffff, 0x42ffffff, 0x10ffffff, 0x005a3838, 0x005a3838, 0x005a3838, 0x005a3838, 0x10ffffff, 0x43ffffff, 0x89ffffff, 0xe1ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf0ffffff, 0x24ffffff, 0x00c7c262, 0x001d1d1d, 0x00242424, 0x00333333,
  0x00333333, 0x001b1b1b, 0x001b1b1b, 0x007b540e, 0x00a45c0b, 0x4fffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xecffffff, 0xddffffff, 0xddffffff, 0xecffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfbffffff, 0x4dffffff, 0x00baa54b, 0x00bbb55d, 0x001b1b1b, 0x001a1a1a, 0x00333333,
  0x00000000, 0x00222222, 0x001b1b1b, 0x00674811, 0x00b0770e, 0x007f0300, 0x65ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x63ffffff, 0x00300502, 0x00e1d361, 0x009b964f, 0x001b1b1b, 0x00222222, 0x00000000,
  0x00000000, 0x001a1a1a, 0x001e1e1e, 0x00372c18, 0x00af7c0f, 0x00a24a09, 0x007a0000, 0x60ffffff, 0xfbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x67ffffff, 0x002b0000, 0x00856428, 0x00e3d764, 0x0049462d, 0x001e1e1e, 0x001a1a1a, 0x00000000,
  0x00000000, 0x00000000, 0x00181818, 0x001d1d1d, 0x00715614, 0x00c69413, 0x009a4408, 0x00770200, 0x51ffffff, 0xf2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf0ffffff, 0x4dffffff, 0x002d0200, 0x007a541b, 0x00ddc550, 0x00928c48, 0x001d1d1d, 0x00181818, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x001a1a1a, 0x001a1a1a, 0x001e1e1e, 0x007a6016, 0x00b38110, 0x00a56d0b, 0x008e4706, 0x26ffffff, 0xcfffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcdffffff, 0x24ffffff, 0x00875000, 0x00b1831b, 0x00c3a134, 0x008a7d3b, 0x001e1e1e, 0x001a1a1a, 0x001a1a1a, 0x00000000, 0x00000000,
  0x00ffffff, 0x00000000, 0x00000000, 0x00151515, 0x001b1b1b, 0x001d1d1d, 0x003a3118, 0x006c4d12, 0x00764d0b, 0x006f4107, 0x04ffffff, 0x6fffffff, 0xf2ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xf4ffffff, 0x80ffffff, 0x06ffffff, 0x00875906, 0x00835808, 0x00765b1c, 0x003f3920, 0x001d1d1d, 0x001b1b1b, 0x00151515, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x001a1a1a, 0x00181818, 0x001e1e1e, 0x001b1b1b, 0x001b1b1b, 0x001d1d1d, 0x001c1c1c, 0x001b1b1b, 0x19ffffff, 0x8dffffff, 0xf4ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xf3ffffff, 0x8bffffff, 0x18ffffff, 0x001b1b1b, 0x001c1c1c, 0x001d1d1d, 0x001b1b1b, 0x001b1b1b, 0x001e1e1e, 0x00181818, 0x001a1a1a, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x002b2b2b, 0x001a1a1a, 0x00222222, 0x001b1b1b, 0x00181818, 0x00212121, 0x00212121, 0x00212121, 0x00212121, 0x12ffffff, 0x6bffffff,
  0xc8ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0xc7ffffff,
  0x6affffff, 0x12ffffff, 0x00212121, 0x00212121, 0x00212121, 0x00212121, 0x00181818, 0x001b1b1b, 0x00222222, 0x001a1a1a, 0x002b2b2b, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b,
  0x002b2b2b, 0x16ffffff, 0x56ffffff, 0x8effffff, 0xb6ffffff, 0xd8ffffff, 0xecffffff, 0xf9ffffff, 0xf8ffffff, 0xecffffff, 0xd8ffffff, 0xb5ffffff, 0x8dffffff, 0x55ffffff, 0x15ffffff, 0x002b2b2b,
  0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x002b2b2b, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,

  // 128x128
  128, 128,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x11ffffff, 0x37ffffff, 0x5fffffff, 0x86ffffff, 0xa7ffffff, 0xbbffffff, 0xc7ffffff, 0xd2ffffff, 0xddffffff, 0xeeffffff, 0xf9ffffff,
  0xf9ffffff, 0xeeffffff, 0xddffffff, 0xd1ffffff, 0xc6ffffff, 0xbbffffff, 0xa6ffffff, 0x86ffffff, 0x5fffffff, 0x37ffffff, 0x10ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333,
  0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333,
  0x0cffffff, 0x49ffffff, 0x8bffffff, 0xc1ffffff, 0xe9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe9ffffff, 0xc0ffffff, 0x8affffff, 0x48ffffff, 0x0cffffff,
  0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333,
  0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x00242424, 0x00242424, 0x00202020, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x06ffffff, 0x43ffffff, 0x84ffffff, 0xc5ffffff,
  0xfbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfaffffff,
  0xc4ffffff, 0x83ffffff, 0x41ffffff, 0x06ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x00202020, 0x00242424, 0x00242424, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333,
  0x002b2b2b, 0x00202020, 0x001c1c1c, 0x00171717, 0x00151515, 0x00121212, 0x00111111, 0x00202020, 0x00202020, 0x001e1e1e, 0x001e1e1e, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x33ffffff, 0x91ffffff, 0xeaffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xeeffffff, 0x99ffffff, 0x3cffffff, 0x01ffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001e1e1e, 0x001e1e1e, 0x00202020, 0x00202020, 0x00111111, 0x00121212, 0x00151515, 0x00171717, 0x001c1c1c, 0x00202020, 0x002b2b2b,
  0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x00242424, 0x001c1c1c,
  0x00171717, 0x00121212, 0x00202020, 0x001b1b1b, 0x00181818, 0x00161616, 0x001f1f1f, 0x001d1d1d, 0x001c1c1c, 0x001b1b1b, 0x001b1b1b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x1cffffff, 0x77ffffff, 0xd9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd8ffffff, 0x80ffffff, 0x23ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001b1b1b, 0x001b1b1b, 0x001c1c1c, 0x001d1d1d, 0x001f1f1f, 0x00161616, 0x00181818, 0x001b1b1b, 0x00202020, 0x00121212, 0x00171717,
  0x001c1c1c, 0x00242424, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x001c1c1c, 0x00151515, 0x00222222,
  0x001b1b1b, 0x00171717, 0x001d1d1d, 0x001a1a1a, 0x00171717, 0x001c1c1c, 0x001b1b1b, 0x001a1a1a, 0x00181818, 0x00181818, 0x00171717, 0x001d1d1d, 0x001d1d1d, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x30ffffff, 0xa2ffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0xb3ffffff, 0x37ffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001d1d1d, 0x001d1d1d, 0x00171717, 0x00181818, 0x00181818, 0x001a1a1a, 0x001b1b1b, 0x001c1c1c, 0x00171717, 0x001a1a1a, 0x001d1d1d, 0x00171717, 0x001b1b1b,
  0x00222222, 0x00151515, 0x001c1c1c, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x00242424, 0x00171717, 0x00121212, 0x001b1b1b, 0x00151515,
  0x001a1a1a, 0x001e1e1e, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00181818, 0x001b1b1b, 0x001a1a1a, 0x00191919, 0x00191919, 0x00181818, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x3cffffff, 0xb3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc4ffffff, 0x4dffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x00181818, 0x00191919, 0x00191919, 0x001a1a1a, 0x001b1b1b, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001e1e1e, 0x001a1a1a,
  0x00151515, 0x001b1b1b, 0x00121212, 0x00171717, 0x00202020, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x00202020, 0x00151515, 0x001e1e1e, 0x00171717, 0x001a1a1a, 0x001d1d1d,
  0x00181818, 0x001a1a1a, 0x001b1b1b, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00191919, 0x001b1b1b, 0x001b1b1b, 0x001b1b1b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x46ffffff, 0xc4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc2ffffff, 0x44ffffff, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001b1b1b, 0x001b1b1b, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00181818, 0x001b1b1b, 0x001a1a1a, 0x00181818,
  0x001d1d1d, 0x001a1a1a, 0x00171717, 0x001e1e1e, 0x00151515, 0x00202020, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x001c1c1c, 0x00141414, 0x001c1c1c, 0x001f1f1f, 0x00171717, 0x00191919, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001b1b1b, 0x001d1c1a, 0x00262219, 0x00242118, 0x00242018, 0x0025221a, 0x0025211a, 0x0024211a, 0x0024201a, 0x1bffffff,
  0xa2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb3ffffff,
  0x1fffffff, 0x0025221a, 0x0025211a, 0x00242019, 0x00242019, 0x00231e17, 0x00231e17, 0x00242018, 0x001d1b19, 0x001b1b1b, 0x001a1a1a, 0x00191919, 0x001b1b1b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x00191919, 0x00171717, 0x001f1f1f, 0x001b1b1b, 0x00141414, 0x001c1c1c, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x001c1c1c, 0x00141414, 0x001b1b1b, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001b1b1b, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x00232018, 0x00674e15, 0x00987213, 0x00b28713, 0x00c19415, 0x00c39412, 0x00bf8e11, 0x00b8860f, 0x00b17f0d, 0x00ac780c, 0x00a5700a, 0x05ffffff, 0x6fffffff, 0xf2ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xf5ffffff, 0x80ffffff, 0x06ffffff, 0x009c6000, 0x009d6100, 0x009e6200, 0x009f6300, 0x00a06400, 0x00a06500, 0x009a6101, 0x00945e03, 0x00845606, 0x005f410d, 0x00231f17, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001b1b1b, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001b1b1b, 0x00141414, 0x001c1c1c, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00202020, 0x00141414, 0x001b1b1b, 0x001b1b1b, 0x001c1c1c, 0x001a1a1a, 0x00191919, 0x001b1b1b, 0x001a1a1a,
  0x001f1c18, 0x007d5911, 0x00ad7c0f, 0x00b6840f, 0x00bd8d11, 0x00c49312, 0x00ca9914, 0x00c59212, 0x00c08c11, 0x00b8840f, 0x00b37d0d, 0x00ab750b, 0x40ffffff, 0xd5ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xd4ffffff, 0x3effffff, 0x009d6100, 0x009e6200, 0x009f6300, 0x00a06400, 0x00a16500, 0x00a26600, 0x00a26600, 0x00a36700, 0x00a46800, 0x00a26701, 0x007c5208, 0x001f1c18,
  0x001a1a1a, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001c1c1c, 0x001b1b1b, 0x001b1b1b, 0x00141414, 0x00202020, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00242424, 0x00151515, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001b1b1b, 0x00443315,
  0x0095650d, 0x00aa760c, 0x00b17d0e, 0x00bb8410, 0x00c28910, 0x00ca8f11, 0x00cf9513, 0x00c98f11, 0x00c2880f, 0x00bc820e, 0x05ffffff, 0x8bffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x88ffffff, 0x04ffffff, 0x009f6300, 0x00a06400, 0x00a16500, 0x00a26600, 0x00a36700, 0x00a36700, 0x00a46800, 0x00a56900, 0x00a66a00, 0x00986303,
  0x00493511, 0x001b1b1b, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x00151515, 0x00202020, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x00171717, 0x001e1e1e, 0x001f1f1f, 0x001c1c1c, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001c1b1a, 0x006c480f, 0x009b6609,
  0x00a36f0b, 0x00ac750c, 0x00b67c0d, 0x00bd830f, 0x00c5780e, 0x00cd550b, 0x00d04308, 0x00cd3106, 0x00cb2604, 0x1dffffff, 0xc4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc9ffffff, 0x21ffffff, 0x008d1c00, 0x008e2100, 0x00922d00, 0x00963d00, 0x009f5a00, 0x00a46800, 0x00a56900, 0x00a66a00, 0x00a76b00,
  0x00a66b00, 0x007c570e, 0x001d1c1a, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x001f1f1f, 0x001e1e1e, 0x00171717, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x001c1c1c, 0x00121212, 0x00171717, 0x00171717, 0x001c1c1c, 0x00191919, 0x001a1a1a, 0x001c1b1a, 0x00805009, 0x00965f08, 0x009e6709,
  0x00a76d0b, 0x00af730c, 0x00bb670b, 0x00cd1e04, 0x00d10000, 0x00d00000, 0x00cf0000, 0x00ce0000, 0x4dffffff, 0xedffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xecffffff, 0x4bffffff, 0x00850000, 0x00840000, 0x00830000, 0x00830000, 0x00891700, 0x009f5600, 0x00a66a00, 0x00a76b00,
  0x00a86c00, 0x00ad7508, 0x00a17414, 0x001d1c1a, 0x001a1a1a, 0x00191919, 0x001c1c1c, 0x00171717, 0x00171717, 0x00121212, 0x001c1c1c, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00242424, 0x00151515, 0x001b1b1b, 0x001b1b1b, 0x00191919, 0x001b1b1b, 0x001b1b1b, 0x001b1b1b, 0x00643e0d, 0x00905806, 0x00986108, 0x00a16609,
  0x00a86c0a, 0x00c13f07, 0x00d10300, 0x00d10000, 0x00d00000, 0x00cf0000, 0x02ffffff, 0x8bffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x88ffffff, 0x02ffffff, 0x00830000, 0x00820000, 0x00810000, 0x00820200, 0x00943700, 0x00a76a00,
  0x00a86c00, 0x00af770a, 0x00b58213, 0x0087681d, 0x001b1b1b, 0x001b1b1b, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001b1b1b, 0x00151515, 0x00242424, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x001c1c1c, 0x00111111, 0x00151515, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x003a2813, 0x00875005, 0x00905806, 0x009a6008, 0x00a85908,
  0x00ca1702, 0x00d20000, 0x00d10000, 0x00d00000, 0x00cf0000, 0x06ffffff, 0xa8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xadffffff, 0x08ffffff, 0x00810000, 0x00800000, 0x00800000, 0x007f0000, 0x00871600,
  0x00a35f00, 0x00af770a, 0x00b68415, 0x00bb8c1e, 0x0052441f, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001d1d1d, 0x00151515, 0x00111111, 0x001c1c1c, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x002b2b2b, 0x00171717, 0x001b1b1b, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001a1a1a, 0x001b1a19, 0x00764306, 0x00895105, 0x00935807, 0x009b5f08, 0x00c91602,
  0x00d20000, 0x00d10000, 0x00d00000, 0x00cf0000, 0x0cffffff, 0xb9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbdffffff, 0x0effffff, 0x00800000, 0x007f0000, 0x007e0000, 0x007e0000,
  0x00861700, 0x00af780c, 0x00b68415, 0x00bd8e1e, 0x00b28e28, 0x00201f1a, 0x001a1a1a, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001b1b1b, 0x00171717, 0x002b2b2b, 0x00000000, 0x00000000, 0x00ffffff,
  0x00000000, 0x00000000, 0x00000000, 0x00202020, 0x00121212, 0x00171717, 0x001e1e1e, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x005c350a, 0x00824903, 0x008b5205, 0x00935807, 0x00b53304, 0x00d20000,
  0x00d10000, 0x00d00000, 0x00d00000, 0x13ffffff, 0xc7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xccffffff, 0x16ffffff, 0x007f0000, 0x007e0000, 0x007d0000,
  0x007c0000, 0x00984106, 0x00b68415, 0x00be9020, 0x00c49b2a, 0x00967d2d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001e1e1e, 0x00171717, 0x00121212, 0x00202020, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00333333, 0x001c1c1c, 0x00202020, 0x001d1d1d, 0x001a1a1a, 0x001b1b1b, 0x001a1a1a, 0x00211c18, 0x007a4103, 0x00854b04, 0x008c5205, 0x009f4b06, 0x00d10300, 0x00d10000,
  0x00d00000, 0x00cf0000, 0x20ffffff, 0xdaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd9ffffff, 0x1fffffff, 0x007d0000, 0x007c0000,
  0x007c0000, 0x007c0300, 0x00ad7013, 0x00be9020, 0x00c59d2b, 0x00c8a534, 0x0026241c, 0x001a1a1a, 0x001b1b1b, 0x001a1a1a, 0x001d1d1d, 0x00202020, 0x001c1c1c, 0x00333333, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x002b2b2b, 0x00171717, 0x001b1b1b, 0x001a1a1a, 0x001c1c1c, 0x00181818, 0x001b1b1b, 0x00482a0d, 0x007e4302, 0x00854a04, 0x008e5305, 0x00c31602, 0x00d10000, 0x00d00000,
  0x00cf0000, 0x15ffffff, 0xd8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdcffffff, 0x17ffffff, 0x007c0000,
  0x007b0000, 0x007a0000, 0x00892106, 0x00bf9222, 0x00c59d2b, 0x00cdaa37, 0x0076672d, 0x001b1b1b, 0x00181818, 0x001c1c1c, 0x001a1a1a, 0x001b1b1b, 0x00171717, 0x002b2b2b, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00242424, 0x00151515, 0x00181818, 0x00171717, 0x001a1a1a, 0x001a1a1a, 0x001c1c1c, 0x00623407, 0x007e4302, 0x00874c04, 0x00964a04, 0x00d10000, 0x00d00000, 0x00cf0000,
  0x0dffffff, 0xcdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcbffffff, 0x0dffffff,
  0x007a0000, 0x007a0000, 0x00790100, 0x00b7821e, 0x00c79f2d, 0x00cdaa37, 0x00aa9238, 0x001c1c1c, 0x001a1a1a, 0x001a1a1a, 0x00171717, 0x00181818, 0x00151515, 0x00242424, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00242424, 0x00121212, 0x00161616, 0x001c1c1c, 0x001d1d1d, 0x001c1c1c, 0x001a1a1a, 0x006d3804, 0x007e4302, 0x00874c04, 0x00a93103, 0x00d00000, 0x00cf0000, 0x06ffffff,
  0xb9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbeffffff,
  0x07ffffff, 0x00790000, 0x00780000, 0x00a25614, 0x00c79f2d, 0x00cdaa37, 0x00c1a53e, 0x001a1a1a, 0x001c1c1c, 0x001d1d1d, 0x001c1c1c, 0x00161616, 0x00121212, 0x00242424, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00202020, 0x00222222, 0x001f1f1f, 0x001b1b1b, 0x001b1b1b, 0x001a1a1a, 0x001b1b1b, 0x00723a02, 0x007f4502, 0x00874c04, 0x00b32603, 0x00cf0000, 0x02ffffff, 0xaaffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xa7ffffff, 0x02ffffff, 0x00770000, 0x00984310, 0x00c79f2d, 0x00ceac39, 0x00cbae40, 0x001b1b1b, 0x001a1a1a, 0x001b1b1b, 0x001b1b1b, 0x001f1f1f, 0x00222222, 0x00202020, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x001c1c1c, 0x00202020, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x00191919, 0x001c1b1a, 0x00783d01, 0x007f4502, 0x00884d04, 0x00b81e02, 0x00cd0000, 0x8bffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf7ffffff, 0xd1ffffff, 0xbbffffff, 0xaeffffff, 0xa3ffffff, 0x98ffffff,
  0x98ffffff, 0xa2ffffff, 0xadffffff, 0xbbffffff, 0xd1ffffff, 0xf7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x87ffffff, 0x00770000, 0x0090350d, 0x00c8a12f, 0x00ceac39, 0x00d5b844, 0x001e1d1b, 0x00191919, 0x001a1a1a, 0x001a1a1a, 0x001d1d1d, 0x00202020, 0x001c1c1c, 0x00000000, 0x00000000,
  0x00000000, 0x00333333, 0x001c1c1c, 0x001e1e1e, 0x001c1c1c, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x00211c18, 0x00793d01, 0x00814603, 0x00884d04, 0x00ba1b02, 0x4dffffff, 0xfeffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf4ffffff, 0xb8ffffff, 0x77ffffff, 0x46ffffff, 0x20ffffff, 0x02ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x02ffffff, 0x20ffffff, 0x47ffffff, 0x78ffffff, 0xb9ffffff, 0xf4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xfeffffff, 0x4affffff, 0x008d2f0c, 0x00c8a12f, 0x00d0ae3a, 0x00d6b944, 0x0028271e, 0x001c1c1c, 0x001a1a1a, 0x00181818, 0x001c1c1c, 0x001e1e1e, 0x001c1c1c, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001c1c1c, 0x001e1e1e, 0x001b1b1b, 0x001e1e1e, 0x001d1d1d, 0x001b1b1b, 0x00221d19, 0x007a3f01, 0x00814603, 0x00884d04, 0x1effffff, 0xedffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xe5ffffff, 0x8affffff, 0x3cffffff, 0x05ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x06ffffff, 0x3dffffff, 0x8cffffff, 0xe6ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xebffffff, 0x1cffffff, 0x00c8a12f, 0x00d0ae3a, 0x00d6b944, 0x0029271f, 0x001b1b1b, 0x001d1d1d, 0x001e1e1e, 0x001b1b1b, 0x001e1e1e, 0x001c1c1c, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001e1e1e, 0x001b1b1b, 0x001d1d1d, 0x001c1c1c, 0x001b1b1b, 0x00221d19, 0x007a3f01, 0x00814603, 0x05ffffff, 0xc6ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfaffffff,
  0xa5ffffff, 0x47ffffff, 0x03ffffff, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040,
  0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00404040, 0x00414141, 0x00292929, 0x001a1a1a, 0x00303030, 0x003c3c3c, 0x03ffffff, 0x48ffffff, 0xa6ffffff,
  0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xc3ffffff, 0x04ffffff, 0x00d0ae3a, 0x00d7bb46, 0x0029271f, 0x001b1b1b, 0x001c1c1c, 0x001d1d1d, 0x001b1b1b, 0x001e1e1e, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001d1d1d, 0x001c1c1c, 0x001a1a1a, 0x00221d19, 0x007a3f01, 0x00824803, 0x8bffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff, 0xa2ffffff, 0x23ffffff,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x1effffff, 0x91ffffff, 0xf6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x86ffffff, 0x00d1b03c, 0x00d7bb46, 0x0029271f, 0x001a1a1a, 0x001c1c1c, 0x001d1d1d, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x00221d19, 0x007b4002, 0x41ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfbffffff, 0x91ffffff, 0x17ffffff, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363d36, 0x0001e101, 0x0001e101, 0x0001e101,
  0x0001e101, 0x0001e101, 0x18ffffff, 0x93ffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x3dffffff, 0x00d8bd48, 0x0029271f, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x00211d19, 0x06ffffff, 0xd6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcbffffff, 0x35ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x2fffffff, 0xc5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd4ffffff, 0x05ffffff, 0x0029271f, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x00211d19, 0x78ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf4ffffff, 0x65ffffff, 0x01ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x02ffffff, 0x67ffffff, 0xf5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x73ffffff, 0x0029271f, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x1dffffff, 0xf3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd7ffffff, 0x2dffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x28ffffff, 0xd2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf2ffffff, 0x1cffffff, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0xaeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x9fffffff, 0x0affffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000f800, 0x0bffffff, 0xa1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xaaffffff, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x46ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7effffff, 0x001c1c1c, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000f800, 0x00000000, 0x00000000, 0x80ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x42ffffff, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0xc3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x68ffffff, 0x005d5959, 0x005a5757, 0x002d2d2d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x00424242, 0x003f3f3f, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00383a38, 0x00005000, 0x00005000, 0x00005000,
  0x00005000, 0x00005000, 0x00005000, 0x00005000, 0x00005000, 0x00005000, 0x00005000, 0x00004d00, 0x00000000, 0x00000000, 0x00000000, 0x61ffffff, 0xfbffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc0ffffff, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x3fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xfcffffff, 0x57ffffff, 0x00ad6d6d, 0x00993a3a, 0x007a2f2f, 0x00615d5d, 0x00474444, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x00484848, 0x00b1b1b1, 0x00d1d1d1, 0x00c5c5c5, 0x008f8f8f, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x008d8d8d, 0x007a7a7a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363d36, 0x0000e800, 0x0000e800, 0x0000e800,
  0x0000e800, 0x0000e800, 0x0000e800, 0x0000e800, 0x0000e800, 0x0000e800, 0x0000e800, 0x0000e800, 0x0000e800, 0x0000e800, 0x00008200, 0x00000000, 0x58ffffff, 0xfcffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3cffffff, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0xbbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0x6effffff, 0x00d4d4d4, 0x00d0cece, 0x00bbb4b4, 0x00a78080, 0x00892121, 0x00695151, 0x004e4b4b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x00c2c2c2, 0x00898989, 0x003c3c3c, 0x003a3a3a, 0x00616161, 0x003a3a3a, 0x00646464, 0x00828282, 0x00505050, 0x003a3a3a, 0x003a3a3a, 0x00686868, 0x005a5a5a, 0x00757575,
  0x008d8d8d, 0x007a7a7a, 0x003b3b3b, 0x006d6d6d, 0x00808080, 0x004a4a4a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x67ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb7ffffff, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x36ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x7dffffff, 0x00bababa, 0x00bababa, 0x00c2c2c2, 0x00cdcdcd, 0x00c0bcbc, 0x009d9191, 0x00891f1f, 0x00666060, 0x00343434, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003f3f3f, 0x00f2f2f2, 0x003d3d3d, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x00868686, 0x006f6f6f, 0x00bcbcbc, 0x00686868, 0x003a3a3a, 0x00e1e1e1, 0x00a5a5a5, 0x00686868,
  0x008d8d8d, 0x007a7a7a, 0x003f3f3f, 0x00878787, 0x00717171, 0x00cacaca, 0x004f4f4f, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x80ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x34ffffff, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0xb0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0x9fffffff, 0x00a3a3a3, 0x009f9f9f, 0x00a0a0a0, 0x00a6a6a6, 0x00b5b5b5, 0x00c8c8c8, 0x00b7b4b4, 0x008c6767, 0x007f2f2f, 0x00666363, 0x00202020, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x00484848, 0x00e2e2e2, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x006f6f6f, 0x00a5a5a5, 0x00c0c0c0, 0x00a1a1a1, 0x07ffffff, 0x28ffffff, 0x33ffffff, 0x44ffffff,
  0x44ffffff, 0x34ffffff, 0x27ffffff, 0x07ffffff, 0x00a9a9a9, 0x00cbcbcb, 0x00868686, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0xa2ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xaeffffff, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x20ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd4ffffff,
  0x0affffff, 0x00909090, 0x008a8a8a, 0x008b8b8b, 0x00939393, 0x00a0a0a0, 0x00b4b4b4, 0x00cdcdcd, 0x008d8686, 0x00862929, 0x006c5959, 0x00373737, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x00e9e9e9, 0x00525252, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x004d4d4d, 0x26ffffff, 0x6bffffff, 0xabffffff, 0xddffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0xdcffffff, 0xabffffff, 0x6affffff, 0x26ffffff, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0bffffff,
  0xd6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x1fffffff, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x7effffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf4ffffff, 0x2affffff,
  0x008d8d8d, 0x007e7e7e, 0x00777777, 0x00787878, 0x00828282, 0x00929292, 0x00a5a5a5, 0x00c0c0c0, 0x00b0afaf, 0x007d4646, 0x00784545, 0x004a4949, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x00757575, 0x00d9d9d9, 0x00979797, 0x0effffff, 0x6effffff, 0xccffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcbffffff, 0x6dffffff, 0x0dffffff, 0x003b3b3b, 0x00363e36, 0x0000ff00, 0x0000ff00, 0x0000ff00,
  0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x2bffffff, 0xf5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7affffff, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x01ffffff, 0xdaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x66ffffff, 0x00979797,
  0x00838383, 0x00737373, 0x00696969, 0x006c6c6c, 0x00767676, 0x00898989, 0x009e9e9e, 0x00b8b8b8, 0x00bdbdbd, 0x00735959, 0x00803232, 0x00615b5b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x10ffffff, 0x83ffffff, 0xefffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeeffffff, 0x80ffffff, 0x10ffffff, 0x001e661e, 0x001e661e, 0x001e661e,
  0x001e661e, 0x001e661e, 0x001e661e, 0x001e661e, 0x001e661e, 0x001e661e, 0x001e661e, 0x001e661e, 0x001e661e, 0x001e661e, 0x00234b23, 0x002a2a2a, 0x002a2a2a, 0x002a2a2a, 0x002a2a2a, 0x002a2a2a,
  0x002a2a2a, 0x69ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd7ffffff, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x39ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcaffffff, 0x02ffffff, 0x00959595,
  0x00808080, 0x00707070, 0x005f5f5f, 0x00666666, 0x00747474, 0x00868686, 0x009c9c9c, 0x00b6b6b6, 0x00c7c7c7, 0x00645757, 0x00842727, 0x006b6767, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x01ffffff, 0x5dffffff, 0xe8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe7ffffff, 0x5cffffff, 0x01ffffff, 0x0026564e,
  0x0026564e, 0x0026564e, 0x0026564e, 0x0026564e, 0x002e3231, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f, 0x002f2f2f,
  0x002f2f2f, 0x02ffffff, 0xc7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x35ffffff, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x96ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x31ffffff, 0x00b1b1b1, 0x00999999,
  0x00858585, 0x00757575, 0x006d6d6d, 0x006f6f6f, 0x00787878, 0x008b8b8b, 0x009f9f9f, 0x00bababa, 0x00b3b3b3, 0x00674646, 0x00803838, 0x005e5959, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x13ffffff, 0xb3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb1ffffff, 0x12ffffff,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x32ffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x92ffffff, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x07ffffff, 0xedffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x91ffffff, 0x00d1d1d1, 0x00bbbbbb, 0x00a2a2a2,
  0x00919191, 0x00838383, 0x007b7b7b, 0x007c7c7c, 0x00878787, 0x00969696, 0x00a8a8a8, 0x00c4c4c4, 0x00969494, 0x00692929, 0x00784c4c, 0x00484848, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x33ffffff,
  0xe3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe1ffffff,
  0x31ffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x95ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff, 0x05ffffff, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x44ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf7ffffff, 0x19ffffff, 0x00b8b2b2, 0x00cbcbcb, 0x00b2b2b2,
  0x00a0a0a0, 0x00959595, 0x00909090, 0x00919191, 0x00989898, 0x00a4a4a4, 0x00bababa, 0x00c7c7c7, 0x004e4848, 0x00801717, 0x00716666, 0x00363636, 0x001a1a1a, 0x001a1a1a, 0x43ffffff, 0xf2ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xf1ffffff, 0x41ffffff, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x18ffffff, 0xf6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3fffffff, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x85ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x92ffffff, 0x00822222, 0x00a18888, 0x00c3c2c2, 0x00c8c8c8,
  0x00b7b7b7, 0x00aaaaaa, 0x00a5a5a5, 0x00a5a5a5, 0x00adadad, 0x00bcbcbc, 0x00cecece, 0x00726c6c, 0x00672828, 0x007d4343, 0x00666262, 0x001b1b1b, 0x001a1a1a, 0x43ffffff, 0xf8ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xf7ffffff, 0x40ffffff, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x96ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x80ffffff, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0xc6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfaffffff, 0x1fffffff, 0x00665858, 0x008c2323, 0x00918383, 0x00a9a5a5,
  0x00cecece, 0x00c7c7c7, 0x00c2c2c2, 0x00c3c3c3, 0x00cacaca, 0x00c3c3c3, 0x00635c5c, 0x005a3636, 0x00862424, 0x00737171, 0x002b2b2b, 0x001a1a1a, 0x34ffffff, 0xf2ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xf0ffffff, 0x30ffffff, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x1effffff, 0xf9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc2ffffff, 0x00333333, 0x00000000,
  0x00000000, 0x0dffffff, 0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa5ffffff, 0x001b1b1b, 0x00585454, 0x00695757, 0x00852222, 0x00855353,
  0x00787272, 0x00898080, 0x00a09e9e, 0x00928d8d, 0x006b6262, 0x004b4444, 0x006e1f1f, 0x00833333, 0x00736e6e, 0x00464545, 0x001a1a1a, 0x13ffffff, 0xe2ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xe0ffffff, 0x11ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0xa7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf9ffffff, 0x0bffffff, 0x00000000,
  0x00000000, 0x4affffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x46ffffff, 0x001a1a1a, 0x001c1c1c, 0x00444242, 0x00686666, 0x007a3d3d,
  0x00842424, 0x007f2424, 0x00713030, 0x00702626, 0x007b1a1a, 0x00842424, 0x00775454, 0x006f6c6c, 0x00363636, 0x001a1a1a, 0x01ffffff, 0xb3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb0ffffff, 0x0000382e, 0x0000382e, 0x0000382e, 0x0000382e, 0x0000382e, 0x0000382e, 0x0000382e, 0x0000382e, 0x0000382e, 0x0000382e, 0x00002821,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x4affffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x47ffffff, 0x00000000,
  0x00000000, 0x8bffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe5ffffff, 0x03ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x002b2b2b, 0x005d5858,
  0x006c6b6b, 0x006e6c6c, 0x00715d5d, 0x00716262, 0x00727171, 0x00706d6d, 0x00565353, 0x00212121, 0x001a1a1a, 0x001a1a1a, 0x5effffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf2ffffff, 0xbbffffff, 0x95ffffff, 0x86ffffff,
  0x87ffffff, 0x95ffffff, 0xbbffffff, 0xf2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x59ffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000b898,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x04ffffff, 0xe7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x88ffffff, 0x00000000,
  0x00000000, 0xc1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8bffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001c1c1c, 0x002b2b2b, 0x003b3b3b, 0x00383838, 0x00252525, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x11ffffff, 0xe9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0xabffffff, 0x45ffffff, 0x04ffffff, 0x00545454, 0x00353535, 0x00393939,
  0x003a3a3a, 0x003b3b3b, 0x00545454, 0x04ffffff, 0x46ffffff, 0xacffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe6ffffff, 0x0fffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000b898,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x8dffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbfffffff, 0x00000000,
  0x00000000, 0xe9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3cffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x82ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc2ffffff, 0x2cffffff, 0x00393939, 0x00393939, 0x00383838, 0x00363636, 0x00393939, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x00383838, 0x00393939, 0x003a3a3a, 0x00353535, 0x2effffff, 0xc4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7fffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000b898,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x3fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe6ffffff, 0x00000000,
  0x10ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf5ffffff, 0x06ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x0dffffff, 0xeeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x82ffffff, 0x02ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x03ffffff, 0x85ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xedffffff, 0x0dffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000b898,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x06ffffff, 0xf4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x10ffffff,
  0x38ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb7ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x6dffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x6affffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x6dffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x6bffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000b898,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0xbaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x35ffffff,
  0x5fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x77ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xccffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x82ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x86ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc9ffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000b898,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x79ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5dffffff,
  0x86ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x46ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x27ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc3ffffff, 0x02ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x03ffffff, 0xc5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x23ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x49ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x83ffffff,
  0xa9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x20ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x6bffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x2cffffff, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f,
  0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x00404040, 0x00292929, 0x001a1a1a, 0x2fffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x67ffffff, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f,
  0x003f3f3f, 0x003f3f3f, 0x00434343, 0x001f1f1f, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x22ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa7ffffff,
  0xb9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf6ffffff, 0x02ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xacffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xaaffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xadffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa9ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x02ffffff, 0xf7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb7ffffff,
  0xc5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd1ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xddffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0x45ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x48ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdbffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x00673535, 0xd3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc4ffffff,
  0xd3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbbffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x07ffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xf1ffffff, 0x04ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x05ffffff, 0xf3ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x06ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x00673535, 0xbeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd1ffffff,
  0xe0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xaeffffff, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545,
  0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x27ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xbbffffff, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545,
  0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0xbeffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x25ffffff, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545,
  0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00454545, 0x00474747, 0x004d4d4d, 0x00623a3a, 0xb1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdeffffff,
  0xedffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa2ffffff, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d,
  0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x36ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0x96ffffff, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d,
  0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x98ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x34ffffff, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d,
  0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x00554545, 0xa4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff,
  0xf9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x95ffffff, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737,
  0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x43ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0x85ffffff, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737,
  0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x85ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x42ffffff, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737,
  0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x00373737, 0x003b3b3b, 0x004d4d4d, 0x00653636, 0x96ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff,
  0xf9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x95ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x42ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0x84ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x85ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x42ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x00663535, 0x96ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff,
  0xedffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa2ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x35ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0x96ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x98ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x34ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x00663535, 0xa4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff,
  0xe0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xafffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x27ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xbcffffff, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d,
  0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003e3e3e, 0x00262626, 0x001a1a1a, 0x002f2f2f, 0x003d3d3d, 0xbeffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x25ffffff, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d, 0x003d3d3d,
  0x003d3d3d, 0x003d3d3d, 0x00404040, 0x001e1e1e, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x00663535, 0xb1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdeffffff,
  0xd3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbcffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x07ffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xf2ffffff, 0x04ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x06ffffff, 0xf4ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfbffffff, 0x06ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x00663535, 0xbeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd1ffffff,
  0xc6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd2ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xddffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0x46ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x49ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdaffffff, 0x000b0b0b, 0x000b0b0b, 0x000b0b0b, 0x000b0b0b, 0x000b0b0b, 0x000b0b0b, 0x000b0b0b,
  0x000b0b0b, 0x001b1b1b, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x00663535, 0xd4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc4ffffff,
  0xbaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf6ffffff, 0x02ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xabffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xabffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0xafffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa8ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x02ffffff, 0xf7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb6ffffff,
  0xa9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x20ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x6affffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x2dffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x31ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x67ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x22ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa7ffffff,
  0x86ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x46ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x26ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc4ffffff, 0x03ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x04ffffff, 0xc7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfeffffff, 0x22ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x49ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x83ffffff,
  0x5effffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x77ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0xcaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x85ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x89ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc7ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0x79ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x5cffffff,
  0x38ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb8ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001b1b1b,
  0x00262626, 0x00323232, 0x00413f3f, 0x003d3c3c, 0x00303030, 0x00222222, 0x001a1a1a, 0x001a1a1a, 0x6cffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x6dffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x6fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x6affffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x004d4d4d, 0xbbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x35ffffff,
  0x10ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf5ffffff, 0x07ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00313131, 0x00555252,
  0x00575555, 0x005f4949, 0x006e3939, 0x006b3e3e, 0x00605050, 0x005d5d5d, 0x00565252, 0x00282828, 0x0dffffff, 0xedffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x86ffffff, 0x03ffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x04ffffff, 0x8affffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xecffffff, 0x0dffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x07ffffff, 0xf6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x0effffff,
  0x00000000, 0xe9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3dffffff, 0x001a1a1a, 0x001a1a1a, 0x00202020, 0x00484444, 0x00584e4e, 0x007c2525,
  0x00a74d4d, 0x00bb7b7b, 0x00cdadad, 0x00c09898, 0x00a85f5f, 0x00943131, 0x00753636, 0x00615e5e, 0x00413f3f, 0x80ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc6ffffff, 0x2fffffff, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x00808080, 0x00707070, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x30ffffff, 0xc7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x7dffffff, 0x0000d800, 0x0000d800, 0x0000d800, 0x00007900, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x40ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe6ffffff, 0x00000000,
  0x00000000, 0xc1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x8cffffff, 0x001a1a1a, 0x001c1c1c, 0x00514d4d, 0x00673c3c, 0x00972e2e, 0x00dfc6c6,
  0x00e3dcdc, 0x00dddbdb, 0x00d9d9d9, 0x00d5d5d5, 0x00cdc9c9, 0x00bbb5b5, 0x00a47070, 0x00852121, 0x00675555, 0x0fffffff, 0xe7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0xadffffff, 0x48ffffff, 0x05ffffff, 0x00434343, 0x003c3c3c, 0x004e4e4e,
  0x008d8d8d, 0x007a7a7a, 0x003a3a3a, 0x05ffffff, 0x49ffffff, 0xaeffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe5ffffff, 0x0effffff, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x00242424, 0x8fffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbfffffff, 0x00000000,
  0x00000000, 0x8affffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe8ffffff, 0x04ffffff, 0x003f3d3d, 0x00604444, 0x00a64444, 0x00ece5e5, 0x00e0dfdf,
  0x00cccccc, 0x00c1c1c1, 0x00bcbcbc, 0x00bdbdbd, 0x00c4c4c4, 0x00cfcfcf, 0x00bab5b5, 0x009d8d8d, 0x00892020, 0x00666262, 0x5bffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf3ffffff, 0xbdffffff, 0x97ffffff, 0x87ffffff,
  0x86ffffff, 0x97ffffff, 0xbeffffff, 0xf4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x58ffffff, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x04ffffff, 0xe7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x87ffffff, 0x00000000,
  0x00000000, 0x49ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x48ffffff, 0x00565353, 0x00891f1f, 0x00e6d9d9, 0x00dcdcdc, 0x00c4c4c4,
  0x00b2b2b2, 0x00a5a5a5, 0x00a0a0a0, 0x00a1a1a1, 0x00a8a8a8, 0x00b7b7b7, 0x00cacaca, 0x00b4b0b0, 0x008d6161, 0x007d3232, 0x01ffffff, 0xb2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xadffffff, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0x4bffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x46ffffff, 0x00000000,
  0x00000000, 0x0cffffff, 0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa6ffffff, 0x00703434, 0x00c68f8f, 0x00e1dddd, 0x00c7c7c7, 0x00aeaeae,
  0x009d9d9d, 0x00929292, 0x008b8b8b, 0x008d8d8d, 0x00949494, 0x00a1a1a1, 0x00b6b6b6, 0x00cdcdcd, 0x008a8484, 0x00872626, 0x006c5b5b, 0x12ffffff, 0xe1ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xddffffff, 0x10ffffff, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x001a1a1a, 0xa8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf9ffffff, 0x0affffff, 0x00000000,
  0x00000000, 0x00333333, 0xc5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfaffffff, 0x20ffffff, 0x00ded0d0, 0x00d6d6d6, 0x00b9b9b9, 0x00a0a0a0,
  0x008e8e8e, 0x007f7f7f, 0x00777777, 0x00797979, 0x00838383, 0x00939393, 0x00a6a6a6, 0x00c1c1c1, 0x00afaeae, 0x007d4444, 0x00774646, 0x00494949, 0x31ffffff, 0xf1ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xefffffff, 0x2dffffff, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x0000ff00, 0x00009000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x00212121, 0x1effffff, 0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc1ffffff, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x83ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x94ffffff, 0x00dbd6d6, 0x00cdcdcd, 0x00afafaf, 0x00989898,
  0x00838383, 0x00737373, 0x006a6a6a, 0x006c6c6c, 0x00777777, 0x008a8a8a, 0x009e9e9e, 0x00b9b9b9, 0x00bcbcbc, 0x00745959, 0x00803434, 0x005f5a5a, 0x001a1a1a, 0x40ffffff, 0xf7ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xf7ffffff, 0x3fffffff, 0x00197119, 0x00197119, 0x00197119, 0x00197119, 0x00197119, 0x00197119, 0x00197119, 0x001f501f, 0x00262626, 0x00262626, 0x00262626, 0x00262626, 0x00262626,
  0x00262626, 0x002d2d2d, 0x00444444, 0x00212121, 0x98ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x80ffffff, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x42ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf7ffffff, 0x1affffff, 0x00cbcbcb, 0x00adadad, 0x00959595,
  0x00808080, 0x006f6f6f, 0x005e5e5e, 0x00666666, 0x00747474, 0x00868686, 0x009c9c9c, 0x00b6b6b6, 0x00c7c7c7, 0x00635757, 0x00842727, 0x006c6868, 0x001a1a1a, 0x001a1a1a, 0x41ffffff, 0xf1ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xefffffff, 0x3fffffff, 0x00304844, 0x00304844, 0x00353737, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535, 0x00353535,
  0x00353535, 0x00373737, 0x00444444, 0x19ffffff, 0xf7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3effffff, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x06ffffff, 0xecffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x93ffffff, 0x00cecece, 0x00b0b0b0, 0x00989898,
  0x00848484, 0x00747474, 0x006c6c6c, 0x006e6e6e, 0x00787878, 0x008a8a8a, 0x009f9f9f, 0x00b9b9b9, 0x00b4b4b4, 0x00664747, 0x00803737, 0x00605b5b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x31ffffff,
  0xe0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdeffffff,
  0x2effffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x00444444, 0x97ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xeaffffff, 0x05ffffff, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x95ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x31ffffff, 0x00bababa, 0x00a1a1a1,
  0x00909090, 0x00818181, 0x00797979, 0x007b7b7b, 0x00858585, 0x00959595, 0x00a7a7a7, 0x00c3c3c3, 0x00999898, 0x00692b2b, 0x00794b4b, 0x004a4949, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x11ffffff, 0xb0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xaeffffff, 0x10ffffff,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00141414, 0x33ffffff, 0xfcffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x91ffffff, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x38ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcaffffff, 0x02ffffff, 0x00b1b1b1,
  0x009f9f9f, 0x00949494, 0x008e8e8e, 0x008f8f8f, 0x00979797, 0x00a3a3a3, 0x00b8b8b8, 0x00c9c9c9, 0x00514b4b, 0x007d1717, 0x00716565, 0x00383838, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x00393939, 0x5affffff, 0xe6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe5ffffff, 0x57ffffff, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x02ffffff, 0xc8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x34ffffff, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0xd8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x68ffffff, 0x00c7c7c7,
  0x00b5b5b5, 0x00a8a8a8, 0x00a3a3a3, 0x00a4a4a4, 0x00ababab, 0x00bababa, 0x00cdcdcd, 0x00797575, 0x00652c2c, 0x007f3f3f, 0x00696565, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x00393939, 0x00626262, 0x00d7d7d7, 0x0fffffff, 0x7effffff, 0xedffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xecffffff, 0x7cffffff, 0x0dffffff, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x6bffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd6ffffff, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x7cffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf7ffffff, 0x2fffffff,
  0x00cecece, 0x00c5c5c5, 0x00c0c0c0, 0x00c1c1c1, 0x00c8c8c8, 0x00c6c6c6, 0x006c6666, 0x00573b3b, 0x00862020, 0x00737171, 0x002d2d2d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x004c4c4c, 0x00848484, 0x00ababab, 0x00bebebe, 0x00aeaeae, 0x0dffffff, 0x6bffffff, 0xc8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc6ffffff, 0x6affffff, 0x0cffffff, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x2cffffff, 0xf5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x79ffffff, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x1fffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd5ffffff,
  0x0bffffff, 0x00938d8d, 0x00a8a8a8, 0x009b9a9a, 0x00766f6f, 0x004d4747, 0x006a2525, 0x00842b2b, 0x00736a6a, 0x004b4a4a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x005d5d5d, 0x00cbcbcb, 0x00737373, 0x008b8b8b, 0x009d9d9d, 0x008d8d8d, 0x007d7d7d, 0x003c3c3c, 0x23ffffff, 0x68ffffff, 0xa9ffffff, 0xdbffffff, 0xfbffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfbffffff, 0xdbffffff, 0xa8ffffff, 0x66ffffff, 0x22ffffff, 0x003b3b3b, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x00001814, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0cffffff,
  0xd8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x1effffff, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0xaeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xa2ffffff, 0x007b2828, 0x00713c3c, 0x006e2e2e, 0x00761c1c, 0x00822020, 0x007a4d4d, 0x00716f6f, 0x003e3e3e, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x00565656, 0x00bdbdbd, 0x00adadad, 0x009d9d9d, 0x008c8c8c, 0x007c7c7c, 0x006c6c6c, 0x00383838, 0x00464646, 0x00bebebe, 0x00b1b1b1, 0x00a1a1a1, 0x06ffffff, 0x24ffffff, 0x33ffffff, 0x44ffffff,
  0x43ffffff, 0x33ffffff, 0x24ffffff, 0x06ffffff, 0x00959595, 0x00848484, 0x00747474, 0x00494949, 0x003b3b3b, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00383939, 0x0000352b, 0x0000352b, 0x0000352b,
  0x0000352b, 0x0000352b, 0x0000352b, 0x0000352b, 0x0000221c, 0x0000201a, 0x0000201a, 0x0000201a, 0x0000201a, 0x0000201a, 0x0000201a, 0x0000201a, 0x0000201a, 0x0000201a, 0x01ffffff, 0xa6ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xacffffff, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x31ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x80ffffff, 0x00745353, 0x00735959, 0x00716d6d, 0x00727171, 0x005d5959, 0x00242424, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x00424242, 0x009a9a9a, 0x009c9c9c, 0x008c8c8c, 0x007b7b7b, 0x006b6b6b, 0x004d4d4d, 0x00353535, 0x003a3a3a, 0x00797979, 0x00a0a0a0, 0x00909090, 0x007f7f7f, 0x006f6f6f, 0x005b5b5b, 0x00353535,
  0x00393939, 0x005b5b5b, 0x00a4a4a4, 0x00949494, 0x00848484, 0x00737373, 0x00636363, 0x00363636, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x84ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x34ffffff, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0xb9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0x6effffff, 0x003f3f3f, 0x002c2c2c, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x00424242, 0x007a7a7a, 0x007b7b7b, 0x006b6b6b, 0x004e4e4e, 0x00353535, 0x003a3a3a, 0x003a3a3a, 0x003f3f3f, 0x006d6d6d, 0x007f7f7f, 0x006f6f6f, 0x00585858, 0x00363636, 0x00393939,
  0x003a3a3a, 0x003b3b3b, 0x005c5c5c, 0x00828282, 0x00737373, 0x00606060, 0x003b3b3b, 0x00383838, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x68ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb5ffffff, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x3effffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xfcffffff, 0x58ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x00393939, 0x003b3b3b, 0x00393939, 0x00363636, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x00393939, 0x003b3b3b, 0x00393939, 0x00353535, 0x00393939, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x00393939, 0x003b3b3b, 0x003b3b3b, 0x00353535, 0x00383838, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x59ffffff, 0xfdffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x3affffff, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0xc2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x69ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x6bffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbeffffff, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x44ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x80ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x01ffffff, 0x84ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x40ffffff, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0xacffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa2ffffff, 0x0bffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x00363e3d, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2,
  0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0000ffd2, 0x0cffffff, 0xa5ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa8ffffff, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x1cffffff, 0xf3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd8ffffff, 0x2effffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00444444,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a,
  0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003a3a3a, 0x003e3e3e, 0x00303030, 0x001a1a1a, 0x003b3b3b, 0x003a3a3a, 0x00334340, 0x00334340, 0x00334340,
  0x00334340, 0x00334340, 0x00334340, 0x00334340, 0x00334340, 0x00334340, 0x00334340, 0x2affffff, 0xd4ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf2ffffff, 0x1bffffff, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x0024201a, 0x75ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf6ffffff, 0x6affffff, 0x02ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00474747,
  0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343,
  0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00434343, 0x00454545, 0x002b2b2b, 0x001a1a1a, 0x00373737, 0x00414141, 0x00414141, 0x00414141, 0x00414141,
  0x00414141, 0x00414141, 0x00414141, 0x00414141, 0x00414141, 0x02ffffff, 0x6cffffff, 0xf6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x71ffffff, 0x002b2a21, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x0024201a, 0x04ffffff, 0xd1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcdffffff, 0x37ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x31ffffff, 0xc7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd3ffffff, 0x05ffffff, 0x002a2a21, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x0024201a, 0x009f6a0a, 0x3fffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfcffffff, 0x95ffffff, 0x19ffffff, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x1affffff, 0x97ffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x3bffffff, 0x00fbf67a, 0x002b2a21, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x001d1d1d, 0x001c1c1c, 0x001a1a1a, 0x0024201a, 0x009f6a0a, 0x00a7710b, 0x89ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf9ffffff, 0xa2ffffff, 0x25ffffff,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x1fffffff, 0x91ffffff, 0xf6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0x84ffffff, 0x00f4eb71, 0x00fcf77c, 0x002b2a21, 0x001a1a1a, 0x001c1c1c, 0x001d1d1d, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001a1a1a, 0x001e1e1e, 0x001b1b1b, 0x001d1d1d, 0x001c1c1c, 0x001b1b1b, 0x0024201a, 0x009f6a0a, 0x00a7710b, 0x04ffffff, 0xc4ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfaffffff,
  0xa7ffffff, 0x49ffffff, 0x03ffffff, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f,
  0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x003f3f3f, 0x04ffffff, 0x4bffffff, 0xa9ffffff,
  0xfbffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xc1ffffff, 0x04ffffff, 0x00f6ed73, 0x00fcf77c, 0x002b2a21, 0x001b1b1b, 0x001c1c1c, 0x001d1d1d, 0x001b1b1b, 0x001e1e1e, 0x001a1a1a, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001c1c1c, 0x001e1e1e, 0x001b1b1b, 0x001e1e1e, 0x001d1d1d, 0x001b1b1b, 0x0024201a, 0x00a06b0a, 0x00a7710b, 0x00af780c, 0x1dffffff, 0xecffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xe7ffffff, 0x8effffff, 0x3fffffff, 0x07ffffff, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d,
  0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x004d4d4d, 0x07ffffff, 0x40ffffff, 0x8fffffff, 0xe8ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xeaffffff, 0x1bffffff, 0x00efe369, 0x00f6ed73, 0x00faf57a, 0x002b2a21, 0x001b1b1b, 0x001d1d1d, 0x001e1e1e, 0x001b1b1b, 0x001e1e1e, 0x001c1c1c, 0x00333333, 0x00000000,
  0x00000000, 0x00333333, 0x001c1c1c, 0x001e1e1e, 0x001c1c1c, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x00231f19, 0x00a06b0a, 0x00a8720c, 0x00af780c, 0x00922804, 0x4bffffff, 0xfeffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf5ffffff, 0xbaffffff, 0x79ffffff, 0x48ffffff, 0x22ffffff, 0x03ffffff, 0x008f0c0c, 0x008f0c0c, 0x008e0c0c, 0x008e0c0c, 0x008d0c0c,
  0x008d0c0c, 0x008d0c0c, 0x008c0c0c, 0x008b0c0c, 0x008b0c0c, 0x03ffffff, 0x22ffffff, 0x48ffffff, 0x7affffff, 0xbbffffff, 0xf6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xfdffffff, 0x48ffffff, 0x0069441e, 0x00efe369, 0x00f7f075, 0x00faf57a, 0x002a2920, 0x001c1c1c, 0x001a1a1a, 0x00181818, 0x001c1c1c, 0x001e1e1e, 0x001c1c1c, 0x00333333, 0x00000000,
  0x00000000, 0x00000000, 0x001c1c1c, 0x00202020, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x00191919, 0x001c1b1a, 0x00a16c0a, 0x00a8720c, 0x00b0790d, 0x00942e05, 0x00810000, 0x88ffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf8ffffff, 0xd4ffffff, 0xbbffffff, 0xafffffff, 0xa3ffffff, 0x98ffffff,
  0x98ffffff, 0xa3ffffff, 0xafffffff, 0xbbffffff, 0xd5ffffff, 0xf8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0x84ffffff, 0x002f0000, 0x00714d22, 0x00f1e56b, 0x00f7f075, 0x00f9f479, 0x001e1e1c, 0x00191919, 0x001a1a1a, 0x001a1a1a, 0x001d1d1d, 0x00202020, 0x001c1c1c, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00202020, 0x00222222, 0x001f1f1f, 0x001b1b1b, 0x001b1b1b, 0x001a1a1a, 0x001b1b1b, 0x009b680b, 0x00a7730c, 0x00b0790d, 0x00993a06, 0x00800000, 0x01ffffff, 0xa1ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xa6ffffff, 0x02ffffff, 0x002e0000, 0x0081612b, 0x00f1e56b, 0x00f7f075, 0x00ece773, 0x001b1b1b, 0x001a1a1a, 0x001b1b1b, 0x001b1b1b, 0x001f1f1f, 0x00222222, 0x00202020, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00242424, 0x00121212, 0x00161616, 0x001c1c1c, 0x001d1d1d, 0x001c1c1c, 0x001a1a1a, 0x0093640c, 0x00a9750c, 0x00b0790d, 0x00a14d09, 0x007f0000, 0x007f0000, 0x05ffffff,
  0xb8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbdffffff,
  0x07ffffff, 0x002d0000, 0x002c0000, 0x009d823a, 0x00f1e56b, 0x00f8f276, 0x00e0db6e, 0x001a1a1a, 0x001c1c1c, 0x001d1d1d, 0x001c1c1c, 0x00161616, 0x00121212, 0x00242424, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00242424, 0x00151515, 0x00181818, 0x00171717, 0x001a1a1a, 0x001a1a1a, 0x001c1c1c, 0x00845b0f, 0x00a9750c, 0x00b17b0e, 0x00b1730d, 0x007f0100, 0x007e0000, 0x007d0000,
  0x0affffff, 0xc6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcbffffff, 0x0cffffff,
  0x002d0000, 0x002c0000, 0x002c0100, 0x00d6c358, 0x00f2e76d, 0x00f8f276, 0x00c5bf61, 0x001c1c1c, 0x001a1a1a, 0x001a1a1a, 0x00171717, 0x00181818, 0x00151515, 0x00242424, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x002b2b2b, 0x00171717, 0x001b1b1b, 0x001a1a1a, 0x001c1c1c, 0x00181818, 0x001b1b1b, 0x005e4313, 0x00aa760c, 0x00b17c0e, 0x00b9830f, 0x008f2404, 0x007d0000, 0x007c0000,
  0x007c0000, 0x14ffffff, 0xd7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xdbffffff, 0x17ffffff, 0x002d0000,
  0x002c0000, 0x002b0000, 0x005b3717, 0x00ecdd64, 0x00f2e76d, 0x00f9f478, 0x00888448, 0x001b1b1b, 0x00181818, 0x001c1c1c, 0x001a1a1a, 0x001b1b1b, 0x00171717, 0x002b2b2b, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00333333, 0x001c1c1c, 0x00202020, 0x001d1d1d, 0x001a1a1a, 0x001b1b1b, 0x001a1a1a, 0x00242019, 0x00a7740d, 0x00b17e0e, 0x00b9830f, 0x00b4740e, 0x007e0501, 0x007c0000,
  0x007b0000, 0x007a0000, 0x1fffffff, 0xd9ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd9ffffff, 0x1fffffff, 0x002d0000, 0x002c0000,
  0x002b0000, 0x00300602, 0x00c9b34d, 0x00ecdd64, 0x00f2e76d, 0x00f4ef76, 0x00292820, 0x001a1a1a, 0x001b1b1b, 0x001a1a1a, 0x001d1d1d, 0x00202020, 0x001c1c1c, 0x00333333, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00202020, 0x00121212, 0x00171717, 0x001e1e1e, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x007b5811, 0x00b27f0e, 0x00b88410, 0x00c08a10, 0x00a44e0a, 0x007b0000,
  0x007a0000, 0x007a0000, 0x00790000, 0x12ffffff, 0xc6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xcbffffff, 0x16ffffff, 0x002d0000, 0x002c0000, 0x002b0000,
  0x002a0000, 0x008c6c2c, 0x00e5d25a, 0x00ecdd64, 0x00f3e96f, 0x00b5b25c, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001e1e1e, 0x00171717, 0x00121212, 0x00202020, 0x00000000, 0x00000000, 0x00000000,
  0x00ffffff, 0x00000000, 0x00000000, 0x002b2b2b, 0x00171717, 0x001b1b1b, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001a1a1a, 0x001c1b19, 0x00a07310, 0x00b88710, 0x00c08b10, 0x00c79112, 0x008b2104,
  0x007a0000, 0x00790000, 0x00780000, 0x00780000, 0x0bffffff, 0xb7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbdffffff, 0x0effffff, 0x002d0000, 0x002c0000, 0x002b0000, 0x002a0000,
  0x004e290f, 0x00dec550, 0x00e5d25a, 0x00eddf66, 0x00dcd466, 0x0022221d, 0x001a1a1a, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001b1b1b, 0x00171717, 0x002b2b2b, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x001c1c1c, 0x00111111, 0x00151515, 0x001d1d1d, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x00493916, 0x00b68510, 0x00c08f12, 0x00c79212, 0x00c08310,
  0x008b2204, 0x00780000, 0x00780000, 0x00770000, 0x00760000, 0x06ffffff, 0xa6ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xa5ffffff, 0x06ffffff, 0x002d0000, 0x002c0000, 0x002b0000, 0x00290000, 0x00502a0e,
  0x00c3a740, 0x00dfc751, 0x00e6d45c, 0x00eadc65, 0x00625f37, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001d1d1d, 0x00151515, 0x00111111, 0x001c1c1c, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00242424, 0x00151515, 0x001b1b1b, 0x001b1b1b, 0x00191919, 0x001b1b1b, 0x001c1c1c, 0x001b1b1b, 0x00846414, 0x00c09012, 0x00c69513, 0x00c89312,
  0x00c18c11, 0x009e4b09, 0x00790400, 0x00760000, 0x00760000, 0x00750000, 0x02ffffff, 0x87ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x84ffffff, 0x01ffffff, 0x002d0000, 0x002c0000, 0x002b0000, 0x002e0501, 0x00825c1c, 0x00d1b13e,
  0x00d8bd48, 0x00e0c953, 0x00e6d45c, 0x00a79f4e, 0x001b1b1b, 0x001b1b1b, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001b1b1b, 0x00151515, 0x00242424, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x001c1c1c, 0x00121212, 0x00171717, 0x00171717, 0x001c1c1c, 0x00191919, 0x001a1a1a, 0x001d1c1a, 0x00a97f13, 0x00c89913, 0x00c79713,
  0x00c08d10, 0x00bb850f, 0x00ab6c0b, 0x00852003, 0x00750000, 0x00740000, 0x00740000, 0x00730000, 0x44ffffff, 0xe8ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xebffffff, 0x49ffffff, 0x002e0000, 0x002d0000, 0x002c0000, 0x002a0000, 0x004e2408, 0x00ac8323, 0x00cca835, 0x00d2b23e,
  0x00dabf49, 0x00e0c953, 0x00ccbc55, 0x001f1e1c, 0x001a1a1a, 0x00191919, 0x001c1c1c, 0x00171717, 0x00171717, 0x00121212, 0x001c1c1c, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x001a1a1a, 0x001e1e1e, 0x001f1f1f, 0x001d1d1d, 0x001b1b1b, 0x00181818, 0x001a1a1a, 0x001e1d1a, 0x008c6e16, 0x00c39513,
  0x00c09011, 0x00ba860f, 0x00b47e0d, 0x00af780c, 0x00a36409, 0x008f3f05, 0x00852d03, 0x007f2202, 0x007a1a01, 0x1cffffff, 0xc3ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc1ffffff, 0x1bffffff, 0x00532100, 0x00592600, 0x00643201, 0x00794707, 0x00a77714, 0x00be9020, 0x00c59d2b, 0x00cca835, 0x00d3b440,
  0x00d7bc48, 0x009e9041, 0x001e1e1b, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x001f1f1f, 0x001e1e1e, 0x00171717, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00242424, 0x00151515, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001b1b1b, 0x004e4018,
  0x00ad8312, 0x00ba880f, 0x00b3810e, 0x00ad770c, 0x00a8710a, 0x00a16809, 0x009b6207, 0x00945b05, 0x008e5304, 0x00874c02, 0x04ffffff, 0x87ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0x85ffffff, 0x04ffffff, 0x00a66a00, 0x00a66a00, 0x00a76b00, 0x00aa6f02, 0x00b17b0d, 0x00b78617, 0x00bf9222, 0x00c59d2b, 0x00cdaa37, 0x00bea23c,
  0x00564d29, 0x001b1b1b, 0x001a1a1a, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x00151515, 0x00202020, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00202020, 0x00141414, 0x001b1b1b, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x00191919, 0x001c1c1c, 0x001a1a1a,
  0x001c1b19, 0x00856413, 0x00af7d0f, 0x00ad790c, 0x00a6720a, 0x00a06a09, 0x009a6207, 0x00945b05, 0x008d5304, 0x00874c02, 0x00814500, 0x00874c02, 0x3dffffff, 0xd3ffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xd1ffffff, 0x3cffffff, 0x00a46800, 0x00a56900, 0x00a66a00, 0x00a76b00, 0x00a76b00, 0x00ab7104, 0x00b17b0d, 0x00b98819, 0x00bf9222, 0x00c39c2d, 0x00927b2e, 0x001d1c1a,
  0x001a1a1a, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001b1b1b, 0x00141414, 0x00202020, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x001c1c1c, 0x00141414, 0x001b1b1b, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001b1b1b, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x00252119, 0x00624913, 0x00865e0e, 0x0091600a, 0x00935e08, 0x00935b05, 0x008c5303, 0x00874c02, 0x00814500, 0x00874c02, 0x008f5404, 0x04ffffff, 0x6fffffff, 0xf1ffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xf4ffffff, 0x80ffffff, 0x06ffffff, 0x00a36700, 0x00a46800, 0x00a56900, 0x00a66a00, 0x00a76b00, 0x00a76c00, 0x00a36c05, 0x00a27210, 0x00956f19, 0x006c561e, 0x0026241b, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001b1b1b, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001b1b1b, 0x00141414, 0x001c1c1c, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x001c1c1c, 0x00141414, 0x001c1c1c, 0x00141414, 0x00171717, 0x00191919, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001b1b1b, 0x001c1b1a, 0x00231e18, 0x00211d17, 0x00211c17, 0x00231f19, 0x00231f19, 0x00242019, 0x0024201a, 0x1affffff,
  0xa2ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xb3ffffff,
  0x1effffff, 0x00252019, 0x00252019, 0x00252019, 0x00252119, 0x00231f17, 0x00231f17, 0x00252018, 0x001d1c19, 0x001b1b1b, 0x001a1a1a, 0x00191919, 0x001b1b1b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x00191919, 0x00171717, 0x00141414, 0x001c1c1c, 0x00141414, 0x001c1c1c, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x00202020, 0x00151515, 0x001e1e1e, 0x00171717, 0x001b1b1b, 0x001d1d1d,
  0x00181818, 0x001a1a1a, 0x001b1b1b, 0x00181818, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00191919, 0x001b1b1b, 0x001b1b1b, 0x001b1b1b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x42ffffff, 0xc1ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xbeffffff, 0x40ffffff, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001b1b1b, 0x001b1b1b, 0x001b1b1b, 0x00191919, 0x001a1a1a, 0x001c1c1c, 0x001a1a1a, 0x00181818, 0x001b1b1b, 0x001a1a1a, 0x00181818,
  0x001d1d1d, 0x001a1a1a, 0x00171717, 0x001e1e1e, 0x00151515, 0x00202020, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x00242424, 0x00171717, 0x00121212, 0x001b1b1b, 0x00151515,
  0x001a1a1a, 0x001e1e1e, 0x001a1a1a, 0x001d1d1d, 0x001a1a1a, 0x00181818, 0x001b1b1b, 0x001a1a1a, 0x00191919, 0x00191919, 0x00181818, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x3bffffff, 0xb3ffffff, 0xfeffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xc4ffffff, 0x42ffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x00181818, 0x00191919, 0x00191919, 0x001a1a1a, 0x001b1b1b, 0x00181818, 0x001a1a1a, 0x001d1d1d, 0x001a1a1a, 0x001e1e1e, 0x001a1a1a,
  0x00151515, 0x001b1b1b, 0x00121212, 0x00171717, 0x00242424, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x001c1c1c, 0x00151515, 0x00222222,
  0x001c1c1c, 0x00171717, 0x001d1d1d, 0x001a1a1a, 0x00171717, 0x001c1c1c, 0x001b1b1b, 0x001a1a1a, 0x00181818, 0x00181818, 0x00171717, 0x001d1d1d, 0x001d1d1d, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x33ffffff, 0xaeffffff, 0xfdffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xfdffffff, 0xacffffff, 0x31ffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001d1d1d, 0x001d1d1d, 0x00171717, 0x00181818, 0x00181818, 0x001a1a1a, 0x001b1b1b, 0x001c1c1c, 0x00171717, 0x001a1a1a, 0x001d1d1d, 0x00171717, 0x001b1b1b,
  0x00222222, 0x00151515, 0x001c1c1c, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x00242424, 0x001c1c1c,
  0x00171717, 0x00121212, 0x00202020, 0x001b1b1b, 0x00181818, 0x00161616, 0x001f1f1f, 0x001d1d1d, 0x001c1c1c, 0x001b1b1b, 0x001b1b1b, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x1cffffff, 0x6fffffff, 0xd0ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xd7ffffff, 0x80ffffff, 0x22ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001b1b1b, 0x001b1b1b, 0x001c1c1c, 0x001d1d1d, 0x001f1f1f, 0x00161616, 0x00181818, 0x001b1b1b, 0x00202020, 0x00121212, 0x00171717,
  0x001c1c1c, 0x00242424, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333,
  0x002b2b2b, 0x00202020, 0x001c1c1c, 0x00171717, 0x00151515, 0x00121212, 0x00111111, 0x00202020, 0x00202020, 0x001e1e1e, 0x001e1e1e, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x2bffffff, 0x88ffffff, 0xe9ffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xeeffffff, 0x99ffffff, 0x3cffffff, 0x01ffffff, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c,
  0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001e1e1e, 0x001e1e1e, 0x00202020, 0x00202020, 0x00111111, 0x00121212, 0x00151515, 0x00171717, 0x001c1c1c, 0x00202020, 0x002b2b2b,
  0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x002b2b2b, 0x00242424, 0x00242424, 0x00202020, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x06ffffff, 0x40ffffff, 0x81ffffff, 0xc3ffffff,
  0xfaffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xf9ffffff,
  0xc2ffffff, 0x80ffffff, 0x3fffffff, 0x05ffffff, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a,
  0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001a1a1a, 0x001c1c1c, 0x001c1c1c, 0x001c1c1c, 0x00202020, 0x00242424, 0x00242424, 0x002b2b2b, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333,
  0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333,
  0x0bffffff, 0x47ffffff, 0x88ffffff, 0xbfffffff, 0xe7ffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff,
  0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xe6ffffff, 0xbfffffff, 0x88ffffff, 0x46ffffff, 0x0bffffff,
  0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333,
  0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00333333, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
  0x00ffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x0fffffff, 0x35ffffff, 0x5dffffff, 0x84ffffff, 0xa6ffffff, 0xb8ffffff, 0xc4ffffff, 0xd0ffffff, 0xddffffff, 0xedffffff, 0xf9ffffff,
  0xf9ffffff, 0xedffffff, 0xddffffff, 0xd0ffffff, 0xc4ffffff, 0xb8ffffff, 0xa5ffffff, 0x84ffffff, 0x5cffffff, 0x35ffffff, 0x0fffffff, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000,
  0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00000000, 0x00ffffff,
  0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff, 0x00ffffff,
};

static constexpr const uint32_t sCardinalX11IconSize = 2 + 16 * 16 + 2 + 48 * 48 + 2 + 128 * 128;

// declaration first, ensuring extern/visiblity status is respected
extern const unsigned long* gCardinalX11Icon;

// definition afterwards
const unsigned long* gCardinalX11Icon = sCardinalX11Icon;
